package org.gcube.portlets.user.geoportaldataviewer.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.gcube.portlets.user.geoportaldataviewer.client.GeoportalDataViewerConstants.LayerType;
import org.gcube.portlets.user.geoportaldataviewer.client.events.QueryDataEvent;
import org.gcube.portlets.user.geoportaldataviewer.client.events.QueryDataEventHandler;
import org.gcube.portlets.user.geoportaldataviewer.client.gis.OpenLayerOSM;
import org.gcube.portlets.user.geoportaldataviewer.client.util.URLUtil;
import org.gcube.portlets.user.geoportaldataviewer.shared.GeoNaDataObject;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.BoundsMap;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.GeoInformationForWMSRequest;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.GeoQuery;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.LayerItem;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.ZAxis;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;


/**
 * The Class LayerManager.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR (francesco.mangiacrapa@isti.cnr.it)
 * 
 * Oct 27, 2020
 */
public class LayerManager {
	
	/** The ol map. */
	private OpenLayerOSM olMap;
	
	/** The layer items. */
	private List<LayerItem> layerItems = new ArrayList<LayerItem>();
	
	private HandlerManager eventBus = new HandlerManager("");

	/**
	 * Instantiates a new layer manager.
	 */
	public LayerManager() {
		bindEvents();
	}
	
	
	private void bindEvents() {
		
		eventBus.addHandler(QueryDataEvent.TYPE, new QueryDataEventHandler() {
			
			@Override
			public void onQueryInteraction(QueryDataEvent queryEvent) {
				
				if(queryEvent.getGeoQuery()!=null) {
					GeoQuery selectDataInfo = queryEvent.getGeoQuery();
					GWT.log("("+selectDataInfo.getX1()+","+selectDataInfo.getY1()+")("+selectDataInfo.getX2()+","+selectDataInfo.getY2()+")");
					
					for (LayerItem layerItem : layerItems){
						//TODO
						isLayerVisible(layerItem);
						
					}
					BoundsMap mapBBOX = new BoundsMap();
					mapBBOX.setCrs(olMap.getProjectionCode());
					mapBBOX.setLowerLeftX(olMap.getExtent().getLowerLeftX());
					mapBBOX.setLowerLeftY(olMap.getExtent().getLowerLeftY());
					mapBBOX.setUpperRightX(olMap.getExtent().getUpperRightX());
					mapBBOX.setUpperRightY(olMap.getExtent().getUpperRightY());

					GeoportalDataViewerServiceAsync.Util.getInstance().getDataResult(layerItems, olMap.getProjectionCode(), mapBBOX, GeoportalDataViewerConstants.MAX_WFS_FEATURES, olMap.getCurrentZoomLevel(), new AsyncCallback<List<GeoNaDataObject>>() {

						@Override
						public void onFailure(Throwable caught) {
						}

						@Override
						public void onSuccess(List<GeoNaDataObject> result) {
						}
					});
				}
			}
		});
	}


	/**
	 * Adds the layer by wms.
	 *
	 * @param layerTitle the layer title
	 * @param layerName the layer name
	 * @param wmsRequest the wms request
	 * @param isBase the is base
	 * @param UUID the uuid
	 */
	public void addLayerByWms(String layerTitle, String layerName, String wmsRequest, boolean isBase, String UUID) {

		boolean displayInLayerSwitcher = false;
		if(isBase)
			displayInLayerSwitcher = true;

		addLayerByWmsRequest(layerTitle, layerName, wmsRequest, isBase, displayInLayerSwitcher, UUID, true);
	}
	
	/**
	 * Adds the layer by wms request.
	 *
	 * @param layerTitle the layer title
	 * @param layerName the layer name
	 * @param wmsRequest the wms request
	 * @param isBase the is base
	 * @param displayInLayerSwitcher the display in layer switcher
	 * @param UUID the uuid
	 * @param onTop the on top
	 */
	public void addLayerByWmsRequest(final String layerTitle, final String layerName, final String wmsRequest, final boolean isBase,final boolean displayInLayerSwitcher, final String UUID, final boolean onTop) {

//		final LayoutContainer westPanel = (LayoutContainer) layersPanel.getParent();
//
//		if(layersPanel.getLayerItems().size()==0)
//			westPanel.mask("Adding..."+layerName, "x-mask-loading");
//		else
//			layersPanel.mask("Adding..."+layerName, "x-mask-loading");

		final LayerType featureType = isBase?LayerType.RASTER_BASELAYER:LayerType.FEATURE_TYPE;
//		Info.display("Adding Layer", layerName);
		GeoportalDataViewerServiceAsync.Util.getInstance().parseWmsRequest(wmsRequest, layerName, new AsyncCallback<GeoInformationForWMSRequest>() {

			@Override
			public void onFailure(Throwable caught) {

				Window.alert(caught.getMessage());
			}

			@Override
			public void onSuccess(GeoInformationForWMSRequest result) {
				
				GWT.log("Add Layer By WMS: "+result.getMapWMSNoStandard() + ", and isNCWMS? "+result.isNcWMS());
				addLayerByWms(featureType, layerTitle, layerName, result.getWmsRequest(), result.getBaseWmsServiceHost(), true, isBase, displayInLayerSwitcher, (ArrayList<String>) result.getStyles().getGeoStyles(), result.getWmsRequest(), false, result.getMapWMSNoStandard(), result.isNcWMS(), UUID, result.getZAxis());
				
			}
		});
	}
	
	
	/**
	 * Adds the layer by wms.
	 *
	 * @param layerType the layer type
	 * @param layerTitle the layer title
	 * @param layerName the layer name
	 * @param layerURL the layer URL
	 * @param mapServerHost the map server host
	 * @param isExternal the is external
	 * @param isBase the is base
	 * @param displayInLayerSwitcher the display in layer switcher
	 * @param styles the styles
	 * @param serverWmsRequest the server wms request
	 * @param onTop the on top
	 * @param wmsNotStandardParams the wms not standard params
	 * @param isNcWms the is nc wms
	 * @param UUID the uuid
	 * @param zAxis the z axis
	 */
	private void addLayerByWms(LayerType layerType, String layerTitle, String layerName, String layerURL, String mapServerHost, boolean isExternal, boolean isBase, boolean displayInLayerSwitcher, ArrayList<String> styles, String serverWmsRequest, boolean onTop, HashMap<String, String> wmsNotStandardParams, boolean isNcWms, String UUID, ZAxis zAxis) {

//		GWT.log("Add addLayerByWms 1");
		LayerItem layerItem = new LayerItem();
		layerItem.setBaseLayer(isBase);
		layerItem.setTitle(layerTitle);
		layerItem.setName(layerName);
		layerItem.setUrl(layerURL);
		layerItem.setMapServerHost(mapServerHost);
		//l.setExternal(isExternal);
		layerItem.setOpacity(1d);
		layerItem.setBuffer(2);
		layerItem.setServerWmsRequest(serverWmsRequest);
		layerItem.setWmsNotStandardParams(wmsNotStandardParams);
		layerItem.setNcWms(isNcWms);
		layerItem.setUUID(UUID);
		layerItem.setZAxis(zAxis);

		switch (layerType) {

		//TODO IMPLEMENT THIS CASE
			case RASTER_BASELAYER:

//				l.setHasLegend(false);
				layerItem.setBaseLayer(true);
				layerItem.setTrasparent(false);
				layerItem.setClickData(false);
				break;

			case FEATURE_TYPE:

				//CASE FEATURE TYPE
				layerItem.setBaseLayer(false);
				layerItem.setClickData(true);
				layerItem.setTrasparent(true);
				break;

		}

		GWT.log("styles "+styles);

		if(styles!=null && styles.size()>0){
			layerItem.setHasLegend(true);
			layerItem.setDefaultStyle(styles.get(0));
			layerItem.setStyle(styles.get(0));
			layerItem.setStyles(styles);
		}else{
			String style = URLUtil.getValueOfParameter("styles", serverWmsRequest);
			if(style!=null){ //CASE OF STYLE ="";
				//TENTATIVE TO GET LEGEND
				layerItem.setHasLegend(true);
			}
		}
		GWT.log("Built layer: "+layerItem);
		layerItems.add(layerItem);
		olMap.addWMSLayer(layerItem.getMapServerHost(), layerName);

//		openLayersMap.addLayerItemByWms(l, displayInLayerSwitcher);
//		layersPanel.addLayerItems(layerItems, onTop);
//		layersPanel.updateLayersOrder();
	}
	
	
	/**
	 * Checks if is layer visible.
	 *
	 * @param layerItem the layer item
	 * @return true, if is layer visible
	 */
	public boolean isLayerVisible(LayerItem layerItem){
		return olMap.isLayerVisible(layerItem.getName());
	}
	
	
	/**
	 * Sets the ol map.
	 *
	 * @param olMap the new ol map
	 */
	public void setOlMap(OpenLayerOSM olMap) {
		this.olMap = olMap;
	}
	
	public HandlerManager getEventBus() {
		return eventBus;
	}
	

	

}
